<?xml version="1.0"?>
<style>

body, h1, h2, h3, h4, h5, h6, pre, li, div {
	line-height: 1.29em;
}

body {
	background-color: white;
	margin:0 auto;
	font-family: "liberation sans", "Myriad ", "Bitstream Vera Sans", "Lucida Grande", "Luxi Sans", "Trebuchet MS", helvetica, verdana, arial, sans-serif;
	font-size:12px;
	max-width:55em;
	color:black;
}

/* desktop styles */
body.desktop {
	margin-left: 26em;
}

body.desktop .book > .toc {
	display:block;
	width:24em;
	height:99%;
	position:fixed;
	overflow:auto;
	top:0px;
	left:0px;
	padding-left:1em;
	background-color:#EEEEEE;
}

.toc {
	line-height:1.35em;
}

.toc .glossary,
.toc .chapter, .toc .appendix {
	margin-top:1em;
}

.toc .part {
	margin-top:1em;
	display:block;
}

span.glossary,
span.appendix {
	display:block;
	margin-top:0.5em;
}

div {
	padding-top:0px;
}

div.section {
	padding-top:1em;
}

p, div.para, div.formalpara {
	padding-top:0px;
	margin-top:0.3em;
	padding-bottom:0px;
	margin-bottom:1em;
}

/*Links*/
a {
	outline: none;
}

a:link {
	text-decoration:none;
	border-bottom: 1px dotted ;
	color:#3366cc;
}

a:visited {
	text-decoration:none;
	border-bottom: 1px dotted ;
	color:#003366;
}

div.longdesc-link {
	float:right;
	color:#999;
}

.toc a, .qandaset a {
	font-weight:normal;
}

/*headings*/
h1, h2, h3, h4, h5, h6 {
	color: #336699;
	margin-top: 0em;
	margin-bottom: 0em;
	background-color: transparent;
}

h1 {
	font-size:2.0em;
}

.titlepage h1.title {
	font-size: 3.0em;
	padding-top: 1em;
	text-align:left;
}

.book > .titlepage h1.title {
	text-align:center;
}

.article > .titlepage h1.title {
	text-align:center;
}

.set .titlepage > div > div > h1.title {
	text-align:center;
}

.producttitle {
	margin-top: 0em;
	margin-bottom: 0em;
	font-size: 3.0em;
	font-weight: bold;
	color: white;
	text-align: center;
	padding: 0.7em;
}

.titlepage .corpauthor {
	margin-top: 1em;
	text-align: center;
}

.section h1.title {
	font-size: 1.6em;
	padding: 0em;
	color: #336699;
	text-align: left;
	background: white;
}

h2 {
	font-size:1.6em;
}


h2.subtitle, h3.subtitle {
	margin-top: 1em;
	margin-bottom: 1em;
	font-size: 1.4em;
	text-align: center;
}

.preface > div > div > div > h2.title {
	margin-top: 1em;
	font-size: 2.0em;
}

.appendix h2 {
	margin-top: 1em;
	font-size: 2.0em;
}



h3 {
	font-size:1.3em;
	padding-top:0em;
	padding-bottom:0em;
}
h4 {
	font-size:1.1em;
	padding-top:0em;
	padding-bottom:0em;
}

h5 {
	font-size:1em;
}

h6 {
	font-size:1em;
}

h5.formalpara {
	font-size:1em;
	margin-top:2em;
	margin-bottom:.8em;
}

.abstract h6 {
	margin-top:1em;
	margin-bottom:.5em;
	font-size:2em;
}

/*element rules*/
hr {
	border-collapse: collapse;
	border-style:none;
	border-top: 1px dotted #ccc;
	width:100%;
	margin-top: 3em;
}

/* web site rules */
ul.languages, .languages li {
	display:inline;
	padding:0em;
}

.languages li a {
	padding:0em .5em;
	text-decoration: none;
}

.languages li p, .languages li div.para {
	display:inline;
}

.languages li a:link, .languages li a:visited {
	color:#444;
}

.languages li a:hover, .languages li a:focus, .languages li a:active {
	color:black;
}

ul.languages {
	display:block;
	background-color:#eee;
	padding:.5em;
}

/*supporting stylesheets*/

/*unique to the webpage only*/
.books {
	position:relative;
}

.versions li {
	width:100%;
	clear:both;
	display:block;
}

a.version {
	font-size:2em;
	text-decoration:none;
	width:100%;
	display:block;
	padding:1em 0em .2em 0em;
	clear:both;
}

a.version:before {
	content:"Version";
	font-size:smaller;
}

a.version:visited, a.version:link {
	color:#666;
}

a.version:focus, a.version:hover {
	color:black;
}

.books {
	display:block;
	position:relative;
	clear:both;
	width:100%;
}

.books li {
	display:block;
	width:200px;
	float:left;
	position:relative;
	clear: none ;
}

.books .html {
	width:170px;
	display:block;
}

.books .pdf {
	position:absolute;
	left:170px;
	top:0px;
	font-size:smaller;
}

.books .pdf:link, .books .pdf:visited {
	color:#555;
}

.books .pdf:hover, .books .pdf:focus {
	color:#000;
}

.books li a {
	text-decoration:none;
}

.books li a:hover {
	color:black;
}

/*products*/
.products li {
	display: block;
	width:300px;
	float:left;
}

.products li a {
	width:300px;
	padding:.5em 0em;
}

.products ul {
	clear:both;
}

/*revision history*/
.revhistory {
	display:block;
}

.revhistory table {
	background-color:transparent;
	border-color:#fff;
	padding:0em;
	margin: 0;
	border-collapse:collapse;
	border-style:none;
}

.revhistory td {
	text-align :left;
	padding:0em;
	border: none;
	border-top: 1px solid #fff;
	font-weight: bold;
}

.revhistory .simplelist td {
	font-weight: normal;
}

.revhistory .simplelist {
	margin-bottom: 1.5em;
	margin-left: 1em;
}

.revhistory table th {
	display: none;
}


/*credits*/
.authorgroup div {
	clear:both;
	text-align: center;
}

h3.author {
	margin: 0em;
	padding: 0em;
	padding-top: 1em;
}

.authorgroup h4 {
	padding: 0em;
	margin: 0em;
	padding-top: 1em;
	margin-top: 1em;
}

.author, 
.editor, 
.translator, 
.othercredit,
.contrib {
	display: block;
}

.revhistory .author {
	display: inline;
}

.othercredit h3 {
	padding-top: 1em;
}


.othercredit {
	margin:0em;
	padding:0em;
}

.releaseinfo {
	clear: both;
}

.copyright {
	margin-top: 1em;
}

/* qanda sets */
.answer {
	margin-bottom:1em;
	border-bottom:1px dotted #ccc;
}

.qandaset .toc {
	border-bottom:1px dotted #ccc;
}

.question {
	font-weight:bold;
}

.answer .data, .question .data {
	padding-left: 2.6em;
}

.answer label, .question label {
	float:left;
	font-weight:bold;
}


/*Lists*/
ul {
	padding-left:1.6em;
	list-style-type: circle;
}

ul ul {
	list-style-type: circle;
}

ol {
	list-style-image:none;
	list-style-type: decimal;
}

ol ol {
	list-style-type: lower-alpha;
}

ol.arabic {
	list-style-type: decimal;
}

ol.loweralpha {
	list-style-type: lower-alpha;
}

ol.lowerroman {
	list-style-type: lower-roman;
}

ol.upperalpha {
	list-style-type: upper-alpha;
}

ol.upperroman {
	list-style-type: upper-roman;
}

dt {
	font-weight:bold;
	margin-bottom:0em;
	padding-bottom:0em;
}

dd {
	margin:0em;
	margin-left:2em;
	padding-top:0em;
	padding-bottom: 1em;
}

li {
	padding-top:0px;
	margin-top:0em;
	padding-bottom:0px;
	margin-bottom:0.4em;
}

li p, li div.para {
	padding-top:0px;
	margin-top:0em;
	padding-bottom:0px;
	margin-bottom:0.3em;
}

/*images*/
img {
	display:block;
	margin: 2em 0;
}

.inlinemediaobject, .inlinemediaobject img {
	display:inline;
	margin:0em;
}

.figure img {
	display:block;
	margin:0;
}

.figure .title {
	margin:0em;
	margin-bottom:2em;
	padding:0px;
}

/*document modes*/
.confidential {
	background-color:#900;
	color:White;
	padding:.5em .5em;
	text-transform:uppercase;
	text-align:center;
}

.longdesc-link {
	display:none;
}

.longdesc {
	display:none;
}

.prompt {
	padding:0em .3em;
}

/*user interface styles*/
.screen .replaceable {
}

.guibutton, .guilabel {
	font-family: "liberation mono", "bitstream vera mono", "dejavu mono", monospace;
	font-weight: bold;
	white-space: nowrap;
}

.example {
	background-color: #ffffff;
	border-left: 3px solid #aaaaaa;
	padding-top: 1em;
	padding-bottom: 0.1em;
}

.example h6 {
	padding-left: 10px;
}

.example-contents {
	padding-left: 10px;
	background-color: #ffffff;
}

.example-contents .para {
/*	 padding: 10px;*/
}

/*terminal/console text*/
.computeroutput, 
.option {
	font-family:"liberation mono", "bitstream vera mono", "dejavu mono", monospace;
	font-weight:bold;
}

.replaceable {
	font-family:"liberation mono", "bitstream vera mono", "dejavu mono", monospace;
	font-style: italic;
}

.command, .filename, .keycap, .classname, .literal {
	font-family:"liberation mono", "bitstream vera mono", "dejavu mono", monospace;
	font-weight:bold;
}

/* no bold in toc */
.toc * {
	font-weight: inherit;
}

pre {
	font-family:"liberation mono", "bitstream vera mono", "dejavu mono", monospace;
	display:block;
	background-color: #f5f5f5;
	color: #000000;
	border: 1px solid #aaaaaa;
	margin-bottom: 0.3em;
	padding:.5em 1em;
	white-space: pre-wrap; /* css-3 */
	white-space: -moz-pre-wrap !important; /* Mozilla, since 1999 */
	white-space: -pre-wrap; /* Opera 4-6 */
	white-space: -o-pre-wrap; /* Opera 7 */
	word-wrap: break-word; /* Internet Explorer 5.5+ */
	font-size: 0.9em;
}

pre .replaceable, 
pre .keycap {
}

code {
	font-family:"liberation mono", "bitstream vera mono", "dejavu mono", monospace;
	white-space: nowrap;
	font-weight:bold;
}

.parameter code {
	display: inline;
	white-space: pre-wrap; /* css-3 */
	white-space: -moz-pre-wrap !important; /* Mozilla, since 1999 */
	white-space: -pre-wrap; /* Opera 4-6 */
	white-space: -o-pre-wrap; /* Opera 7 */
	word-wrap: break-word; /* Internet Explorer 5.5+ */
}


div.warning, div.note, div.important {
	color: black;
	margin: 0em;
	padding: 0em;
	background: none;
	background-color: white;
	margin-bottom: 1em;
	padding-left: 1em;
	border-left: 2px solid #aaaaaa;
}

div.warning h2, div.note h2,div.important h2 {
	margin: 0em;
	padding: 0em;
	color: #eeeeec;
	padding-top: 0px;
	padding-bottom: 0px;
	height: 1.4em;
	line-height: 1.4em;
	font-size: 1.4em;
	display:inline;
}

div.admonition_header {
	clear: both;
	margin: 0em;
	padding: 0em;
	margin-top: -3.3em;
	padding-left: 58px;
	line-height: 1.0em;
	font-size: 1.0em;
}


div.warning p, div.warning div.para,
div.note p, div.note div.para,
div.important p, div.important div.para {
	padding: 0em;
	margin: 0em;
}

div.admonition {
	border: none;
	border-left: 1px solid #aaaaaa;
	border-right: 1px solid #aaaaaa;
	padding:0em;
	margin:0em;
	padding-top: 1.5em;
	padding-bottom: 1em;
	padding-left: 2em;
	padding-right: 1em;
	background-color: #eeeeec;
	-moz-border-radius: 0px;
	-webkit-border-radius: 0px;
	border-radius: 0px;
}

/*Page Title*/
#title  {
	display:block;
	height:45px;
	padding-bottom:1em;
	margin:0em;
}

#title a.left{
	display:inline;
	border:none;
}

#title a.left img{
	border:none;
	float:left;
	margin:0em;
	margin-top:.7em;
}

#title a.right {
	padding-bottom:1em;
}

#title a.right img {
	border:none;
	float:right;
	margin:0em;
	margin-top:.7em;
}

/*Table*/
table {
	border:1px solid #6c614b;
	width:100%;
	border-collapse:collapse;
}

table.simplelist, .calloutlist table {
	border-style: none;
}

table th {
	text-align:left;
	background-color:#6699cc;
	padding:.3em .5em;
	color:white;
}

table td {
	padding:.15em .5em;
}

table tr.even td {
	background-color:#f5f5f5;
}

table th p:first-child, table td p:first-child, table  li p:first-child,
table th div.para:first-child, table td div.para:first-child, table  li div.para:first-child {
	margin-top:0em;
	padding-top:0em;
	display:inline;
}

th, td {
	border-style:none;
	vertical-align: top;
	border: 1px solid #000;
}

.simplelist th, .simplelist td {
	border: none;
}

table table td {
	border-bottom:1px dotted #aaa;
	background-color:white;
	padding:.6em 0em;
}

table table {
	border:1px solid white;
}

td.remarkval {
	color:#444;
}

td.fieldval {
	font-weight:bold;
}

.lbname, .lbtype, .lbdescr, .lbdriver, .lbhost {
	color:white;
	font-weight:bold;
	background-color:#999;
	width:120px;
}

td.remarkval {
	width:230px;
}

td.tname {
	font-weight:bold;
}

th.dbfield {
	width:120px;
}

th.dbtype {
	width:70px;
}

th.dbdefault {
	width:70px;
}

th.dbnul {
	width:70px;
}

th.dbkey {
	width:70px;
}

span.book {
	margin-top:4em;
	display:block;
}

span.chapter {
	display:block;
	margin-top:0.5em;
}

table.simplelist td, .calloutlist table td {
	border-style: none;
}

/*Breadcrumbs*/
#breadcrumbs ul li.first:before {
	content:" ";
}

#breadcrumbs {
	color:#900;
	padding:3px;
	margin-bottom:25px;
}

#breadcrumbs ul {
	margin-left:0;
	padding-left:0;
	display:inline;
	border:none;
}

#breadcrumbs ul li {
	margin-left:0;
	padding-left:2px;
	border:none;
	list-style:none;
	display:inline;
}

#breadcrumbs ul li:before {
	content:"\0020 \0020 \0020 \00BB \0020";
	color:#333;
}

/*index*/
.glossary h3, 
.index h3 {
	font-size: 2em;
	color:#aaa;
	margin:0em;
}

.indexdiv {
	margin-bottom:1em;
}

.glossary dt,
.index dt {
	color:#444;
	padding-top:.5em;
}

.glossary dl dl dt, 
.index dl dl dt {
	color:#777;
	font-weight:normal;
	padding-top:0em;
}

.index dl dl dt:before {
	content:"- ";
	color:#ccc;
}

/*changes*/
.footnote {
	font-size: .7em;
	margin:0em;
	color:#222;
}

table .footnote {
}

sup {
	color:#999;
	margin:0em;
	padding:0em;
	line-height: .4em;
	font-size: 1em;
	padding-left:0em;
}

.footnote {
	position:relative;
}

.footnote sup  {
	color:#e3dcc0;
	position:absolute;
	left: .4em;
}

.footnote sup a:link, 
.footnote sup a:visited {
	color:#92917d;
	text-decoration:none;
}

.footnote:hover sup a {
	text-decoration:none;
}

.footnote p,.footnote div.para {
	padding-left:2em;
}

.footnote a:link, 
.footnote a:visited {
	color:#00537c;
}

.footnote a:hover {
}

/**/
div.chapter {
	margin-top:3em;
}

div.section {
	margin-top:1em;
}

div.note .replaceable, 
div.important .replaceable, 
div.warning .replaceable, 
div.note .keycap, 
div.important .keycap, 
div.warning .keycap
{
}

ul li p:last-child, ul li div.para:last-child {
	margin-bottom:0em;
	padding-bottom:0em;
}


/* Dirty EVIL Mozilla hack for round corners */
pre {
	-moz-border-radius:11px;
	-webkit-border-radius:11px;
	border-radius: 11px;
}

.example {
	-moz-border-radius:0px;
	-webkit-border-radius:0px;
	border-radius: 0px;
}

.package, .citetitle {
	font-style: italic;
}

.titlepage .edition {
	color: #336699;
	background-color: transparent;
	margin-top: 1em;
	margin-bottom: 1em;
	font-size: 1.4em;
	font-weight: bold;
	text-align: center;
}

span.remark {
	background-color: #ff00ff;
}

.foreignphrase {
	font-style: inherit;
}

dt {
	clear:both;
}

dt img {
	border-style: none;
	max-width: 112px;
}

dt object {
	max-width: 112px;
}

dt .inlinemediaobject, dt object {
	display: inline;
	float: left;
	margin-bottom: 1em;
	padding-right: 1em;
	width: 112px;
}

dl:after {
	display: block;
	clear: both;
	content: "";
}

.toc dd {
	padding-bottom: 0em;
	margin-bottom: 1em;
	padding-left: 1.3em;
	margin-left: 0em;
}

div.toc > dl > dt {
	padding-bottom: 0em;
	margin-bottom: 0em;
	margin-top: 1em;
}


.strikethrough {
	text-decoration: line-through;
}

.underline {
	text-decoration: underline;
}

.calloutlist img, .callout {
	padding: 0em;
	margin: 0em;
	width: 12pt;
	display: inline;
	vertical-align: middle;
}

.stepalternatives {
	list-style-image: none;
	list-style-type: none;
}


a:link {
	color:#0066cc;
}

a:hover, a:active {
	color:#003366;
}

a:visited {
	color:#6699cc;
}


h1 {
	color:#3c6eb4
}

.section h1.title {
	color:#3c6eb4;
}


h2,h3,h4,h5,h6 {
	color:#3c6eb4;
}

table {
	border:1px solid #3c6eb4;
}

table th {
	background-color:#3c6eb4;
}


table tr.even td {
	background-color:#f5f5f5;
}

.revhistory table th {
	color:#3c6eb4;
}

.titlepage .edition {
	color: #3c6eb4;
}

</style>